#ifndef FINDBOARD_H_
#define FINDBOARD_H_

/** this function look up for boards device in the system and open the devices  
* 
* opened device filedescriptors are stored in the dynamically created cards array. 
* It is the responsability of the caller to free() it
*  
* @param cards A pointer to the card area to fill
*
* @retval 0 no cards found 
* @retval -1 system error (errno gives indication) 
* @retval otherwise, number of board found
*/
int apci_find_boards(const char* boardname, int ** boards);

#endif /*FINDBOARD_H_*/
