#ifndef __APCIE1532_KNOWNDEV_H__
#define __APCIE1532_KNOWNDEV_H__

void apcie1532_known_dev_append(struct pci_dev * pdev);
void apcie1532_known_dev_remove(struct pci_dev * pdev);

#endif // __APCIE1532_KNOWNDEV_H__
