#ifndef __APCIE1532_EVENTS_H__
#define __APCIE1532_EVENTS_H__

typedef enum
{
	APCIE1532_EVT_FROM_DIGITAL_INPUT = 0,
	APCIE1532_EVT_FROM_COUNTER = 1,
	APCIE1532_EVT_FROM_DIGOUT = 2,
	APCIE1532_EVT_FROM_TIMER = 4
} event_source_t;

typedef struct
{
	event_source_t source; /**< the subsystem that generated the event */
	unsigned long param1; /**< for APCIE1532_EVT_FROM_DIGITAL_INPUT,APCIE1532_EVT_FROM_COUNTER, this is the bitmask of digital inputs that generated the event */
	unsigned long param2; /**< for APCIE1532_EVT_FROM_DIGOUT, this is the cause of the interrupt  CC or VCC ) */
} apcie1532_event_t;


#endif /*__APCIE1532_EVENTS_H__*/
